<?php

namespace App\Http\Controllers;

use App\Models\Category;
use App\Models\Preference;
use Illuminate\Http\Request;

class PreferencesController extends Controller
{
    /**
     * preferenceList
     *
     * @return void
     */
    function preferenceList()
    {
        $vehicleData = Preference::orderby('created_at', 'desc')->get();
        return view('Preference.PreferencesList', compact('vehicleData'));
    }

    /**
     * preferenceForm
     *
     * @return void
     */
    function preferenceForm()
    {
        // return "hello";
        return view('Preference.PreferencesForm');
    }
    /**
     * preferenceAdd
     *
     * @param  mixed $request
     * @return void
     */
    function preferenceAdd(Request $request)
    {
        $request->validate([
            'title' => 'required|unique:preferences,title|regex:/^[a-zA-Z0-9 ]+$/',
        ]);
        $slug = str_replace(' ', '-', $request->title);
        $request['slug'] = strtolower($slug);
        Preference::create($this->attributes($request));
        return redirect(route('preference.list'))->with('message', 'preference Added Successfully');
    }

    function deletePreference(Preference $preference)
    {
        $preference->delete();
        return redirect(Route('preference.list'))->with('message', 'Preference deleted successfully');
    }
    function preferenceEdit(Preference $Preference)
    {
        return view('Preference.PreferencesForm', compact('Preference'));
    }

    // function updatePreference(Request $request, Preference $preference)
    // {
    //     $request->validate([
    //         "title" => "required"
    //     ]);
    //     $slug = str_replace(' ', '-', $request->title);
    //     $request['slug'] = strtolower($slug);
    //     $preference->update($this->attributes($request));
    //     return redirect(Route('preference.list'))->with('message', 'Preference updated successfully');
    // }
    public function updatePreference(Request $request, Preference $preference)
    {
        $request->validate([
            'title' => 'required|regex:/^[a-zA-Z0-9 ]+$/|unique:preferences,title,' . $preference->id,
        ]);

        $slug = str_replace(' ', '-', $request->title);
        $request->merge(['slug' => strtolower($slug)]);
        $preference->update($this->attributes($request));

        return redirect(Route('preference.list'))->with('message', 'Preference updated successfully');
    }


    private function attributes($request)
    {
        $data = $request->only([
            'title',
            'slug',
        ]);
        return $data;
    }
}
