<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Stripe\StripeClient;

class booking extends Model
{
    use HasFactory;

    protected $fillable = [

        'user_id',
        'nanny_id',
        'service_status',
        'service_id',
        'service_slug',
        'vehicle_category_id',
        'pickup_location',
        'pickup_lat',
        'pickup_lng',
        'dropoff_location',
        'dropoff_lat',
        'dropoff_lng',
        'receiver_details',
        'description',
        'tax_and_charges',
        'discounts',
        'sub_amount',
        'tax_amount',
        'charge_amount',
        'discount_amount',
        'total_amount',
        'payment_method',
        'payment_status',
        'transaction_reference',
        'transaction_id',
        'paid_from',
        'verification_code',
        'cancel_reason',
    ];


    protected $casts = [
        'receiver_details' => 'array'
    ];

    protected $appends = [
        'is_rated',
    ];

    public function getIsRatedAttribute()
    {
        $rate = Rating::where(['booking_id' => $this->id, 'user_id' => auth()->user()->id])->first();
        return $rate != null ? true : false;
    }


    public function charge($card, $return_url, $customer = null)
    {

        if ($this->total_amount == 0) return $this; // No need to charge

        $stripe = new StripeClient(env('STRIPE_SECRET'));
        if (!$customer) $customer = auth()->user();

        if ($this->transaction_id == null) {

            $intent = $stripe->paymentIntents->create([
                "amount" => round($this->total_amount * 100), // amount in cents
                "currency" => 'USD',
                "payment_method_types" => ['card'],
                "payment_method_options" => ['card' => ['setup_future_usage' => 'off_session']],
                "description" => "Order #$this->order_id | $this->created_at",
                "customer" => $customer['customer_stripe_id'],
                "metadata" => [
                    'ordder_id' => $this->id,
                ]
            ])->confirm([
                'payment_method' => $card,
                'return_url' => $return_url
            ]);
        } else {

            $intent = $stripe->paymentIntents->retrieve($this->transaction_id, []);
            $intent->confirm([
                'payment_method' => $card,
                'return_url' => $return_url
            ]);
        }

        $this->update([
            'transaction_id' => $intent->id,
            'paid_from' => 'card',
            'payment_status' => $intent->status
        ]);

        $this->transaction_id = $intent->id;
        $this->payment_status = $intent->status;
        //dd($intent->next_action);
        $this->redirect_url = @$intent->next_action->redirect_to_url->url;
        return $this;
    }


    public function user()
    {
        return $this->belongsTo(User::class, 'user_id');
    }

    public function nanny()
    {
        return $this->belongsTo(User::class, 'nanny_id');
    }
    public function uservehicle()
    {
        return $this->belongsTo(VehicleCategory::class, 'vehicle_category_id');
    }

    public function bookingDetails()
    {
        return $this->hasOne(BookingDetails::class);
    }

    public function service()
    {
        return $this->belongsTo(Service::class);
    }
}
