<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;

use App\Helpers\Helper;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;
use Illuminate\Support\Str;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'first_name',
        'last_name',
        'full_name',
        'email',
        'social_id',
        'provider',
        'phone',
        'password',
        'gender',
        'dob',
        'profile_image',
        'customer_stripe_id',
        'user_type',
        'is_online',
        'service_id',
        'vehicle_category_id',
        'latitude',
        'longitude',

    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    /**
     * appends
     *
     * @var array
     */
    // protected $appends = [
    //     'rating',
    // ];

    // public function getRatingAttribute()
    // {
    //     $user_rating = $this::withCount(['rating as average_rating' => function ($query) {
    //         $query->select(\DB::raw('coalesce(avg(rating),0)'));
    //         //$query->where('user_id', $this->id);
    //     }])->first();
    //     return $user_rating->average_rating;
    // }


    /**
     * The booted method of the model
     */
    protected static function booted()
    {
        static::creating(function ($user) {
            $user->verification_code = mt_rand(10000, 99999);
            $user->phone_verification_code = mt_rand(10000, 99999);
            $user->expired_at = Carbon::now()->addMinutes(60);
        });
    }


    public function verifyUser()
    {
        $this->phone_verified_at = now();
        $this->phone_verification_code = NULL;
        $this->save();
    }

    /**
     * Set password encryption when insert
     *
     * @param $password
     */
    // public function setPasswordAttribute($password)
    // {
    //     $this->attributes['password'] = bcrypt($password);
    // }

    /**
     * Get formatted timestamp of email_verified_at
     *
     * @param $date
     * @return string
     */
    public function getExpiredAtAttribute($date)
    {
        if ($date !== null) {
            return Carbon::parse($date)->format('Y-m-d H:i:s');
        }
    }

    /**
     * Mark user verified
     */
    public function markUserVerified()
    {
        $this->verification_code = null;
        $this->expired_at = null;
        $this->email_verified_at = now();

        $this->save();
    }

    /**
     * Check if verification code is not expired
     *
     * @return bool
     */
    public function hasValidCode()
    {
        return $this->verification_code !== null && Carbon::parse($this->expired_at)->addMinutes(60) > now();
    }

    public function card()
    {
        return $this->hasMany(UserCard::class);
    }
    public function devices()
    {
        return $this->hasMany(UserDevice::class);
    }

    public function helpAndSupport()
    {
        return $this->hasMany(HelpAndSupport::class);
    }

    public function userMedia()
    {
        return $this->hasMany(UserMedia::class);
    }

    public function userVehicle()
    {
        return $this->hasOne(UserVehicle::class);
    }

    public function services()
    {
        return $this->hasMany(UserService::class);
    }

    public function bookings()
    {
        return $this->hasMany(Booking::class);
    }
    public function nannyBookings()
    {
        return $this->hasMany(Booking::class, 'nanny_id');
    }

    public function rating()
    {
        return $this->hasMany(Rating::class);
    }

    public function wallet()
    {
        return $this->hasOne(Wallet::class);
    }

    public function userDevice($request, $access_token)
    {
        $request['access_token'] = $access_token;
        $this->devices()->updateOrCreate(
            $request->only('udid'),
            $request->all()
        );
    }

    public function media($request)
    {
        $extension = $request->file('profile_image')->getClientOriginalExtension();

        return $request->file('profile_image')->storeAs(
            'profile_image/' . Helper::encode(auth()->user()->id),
            Str::random(10) . '.' . $extension,
            env('DISK')
        );
    }

    public function activity()
    {
        return $this->hasMany(Activity::class);
    }
}
