<?php

namespace App\Models;

use Exception;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Wallet extends Model
{
    use HasFactory, SoftDeletes;

    protected $fillable = [
        'user_type',
        'user_id',
        'amount',
        'currency'
    ];

    public function user()
    {
        return $this->belongsTo('user');
    }

    public function transactions()
    {
        return $this->hasMany(WalletTransaction::class);
    }

    public function deposit($amount, $type)
    {
        $wallet = self::updateOrCreate(
            [
                'user_id' => $this->user_id,
            ],
            [
                'amount' => $this->amount + $amount,
                "currency" => "USD",
            ]
        );

        return $this->transactions()->create([
            'amount' => $amount,
            'type' => 'credit',
            'amount_type' => $type
        ]);
    }

    public function withdraw($amount, $type)
    {
        if ($amount > $this->amount) throw new Exception('Insufficient funds.');

        self::update([
            'amount' => $this->amount - $amount
        ]);

        return $this->transactions()->create([
            'amount' => $amount,
            'type' => 'withdrawl',
            'amount_type' => $type
        ]);
    }

    public function penalty($amount, $type)
    {
        $wallet = self::updateOrCreate(
            [
                'user_id' => $this->user_id,
            ],
            [
                'amount' => $this->amount - $amount,
                "currency" => "USD",
            ]
        );

        return $this->transactions()->create([
            'amount' => $amount,
            'type' => 'debit',
            'amount_type' => $type
        ]);
    }
}
